/*
 * Garmin HUD Companion Application
 * Copyright (C) 2022 Piotr Dec / ztsh.eu
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package eu.ztsh.garmin.data

import com.mapbox.navigation.core.trip.session.LocationMatcherResult
import com.mapbox.navigation.core.trip.session.NavigationSessionState
import com.mapbox.navigation.tripdata.maneuver.model.Lane
import com.mapbox.navigation.tripdata.maneuver.model.Maneuver
import com.mapbox.navigation.tripdata.maneuver.model.PrimaryManeuver
import com.mapbox.navigation.tripdata.maneuver.model.StepDistance

class DataCache {

    private var maneuverCache: Maneuver? = null
    private var locationCache: LocationMatcherResult? = null
    private var session: NavigationSessionState? = null
    private var eta: Arrival? = null

    // maneuver
    fun hasChanged(guidance: Lane?): Boolean {
        return guidance != null && maneuverCache.let { it == null || it.laneGuidance != guidance }
    }

    fun hasChanged(distance: StepDistance): Boolean {
        return maneuverCache.let { it == null || it.stepDistance != distance }
    }

    fun hasChanged(primaryManeuver: PrimaryManeuver): Boolean {
        return maneuverCache.let { it == null || it.primary != primaryManeuver }
    }

    fun update(maneuver: Maneuver) {
        maneuverCache = maneuver
    }

    // eta
    fun hasChanged(eta: Arrival): Boolean {
        return this.eta.let { it == null || it != eta }
    }

    fun update(eta: Arrival) {
        this.eta = eta
    }

    // location
    fun hasChanged(locationMatcherResult: LocationMatcherResult): Boolean {
        return locationCache.let { it == null
                || it.enhancedLocation.speed != locationMatcherResult.enhancedLocation.speed
                || it.speedLimitInfo.speed != locationMatcherResult.speedLimitInfo.speed }
    }

    fun update(locationMatcherResult: LocationMatcherResult) {
        locationCache = locationMatcherResult
    }

    // session
    fun isActive(): Boolean {
        return session != null && session is NavigationSessionState.ActiveGuidance
    }

    fun update(sessionState: NavigationSessionState) {
        session = sessionState
    }

}
