/*
 * Garmin HUD Companion Application
 * Copyright (C) 2022 Piotr Dec / ztsh.eu
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package eu.ztsh.garmin.mapbox

import kotlinx.coroutines.delay
import kotlinx.coroutines.runBlocking
import java.util.concurrent.CopyOnWriteArraySet
import java.util.concurrent.atomic.AtomicReference

class NavigationStatusControl {

    private val stateObservers = CopyOnWriteArraySet<NavigationStatusObserver>()
    private val current = AtomicReference(NavigationStatus.IDLE)

    fun registerObserver(observer: NavigationStatusObserver) {
        stateObservers.add(observer)
        observer.onNavigationStatusChanged(current.get())
    }

    fun unregisterObserver(observer: NavigationStatusObserver) {
        stateObservers.remove(observer)
    }

    fun sendEvent(status: NavigationStatus) {
        current.set(status)
        stateObservers.forEach {
            it.onNavigationStatusChanged(status)
        }
        if (status == NavigationStatus.FINISHED || status == NavigationStatus.CANCELED) {
            // TODO: lifecyclescope?
            runBlocking {
                delay(1000)
                sendEvent(NavigationStatus.IDLE)
            }
        }
    }

}

enum class NavigationStatus {
    IDLE,
    STARTED,
    FINISHED,
    CANCELED
}

fun interface NavigationStatusObserver {

    fun onNavigationStatusChanged(navigationStatus: NavigationStatus)

}
