/*
 * Garmin HUD Companion Application
 * Copyright (C) 2022 Piotr Dec / ztsh.eu
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package eu.ztsh.garmin.mapbox

import com.mapbox.api.directions.v5.models.DirectionsRoute
import com.mapbox.navigation.core.MapboxNavigation
import eu.ztsh.garmin.MainActivity
import eu.ztsh.garmin.mock.ReplayResources

class TripControl(private val mapControl: MapControl, private val tripMode: TripMode) {

    val replay = ReplayResources(mapControl)

    fun onAttached(mapboxNavigation: MapboxNavigation) {
        when (tripMode) {
            TripMode.REPLAY -> replay.onAttached(mapboxNavigation)
            TripMode.GPS -> startSession(mapboxNavigation)
        }
    }

    fun onDetached(mapboxNavigation: MapboxNavigation) {
        when (tripMode) {
            TripMode.REPLAY -> replay.onDetached(mapboxNavigation)
            TripMode.GPS -> stopSession(mapboxNavigation)
        }
    }

    fun initLocation() {
        if (tripMode == TripMode.REPLAY) {
            replay.replayOriginLocation()
        }
    }

    fun startSession(mapboxNavigation: MapboxNavigation) {
        if (tripMode == TripMode.GPS) {
            (mapControl.context as MainActivity).permissionsHelper.checkPermissions {
                mapboxNavigation.startTripSession()
            }
        }
    }

    fun startSession(directionsRoute: DirectionsRoute) {
        if (tripMode == TripMode.REPLAY) {
            replay.startSimulation(directionsRoute)
        }
    }

    fun stopSession() {
        if (tripMode == TripMode.REPLAY) {
            replay.stopSimulation()
        }
    }

    fun stopSession(mapboxNavigation: MapboxNavigation) {
        if (tripMode == TripMode.GPS) {
            mapboxNavigation.stopTripSession()
        }
    }

}

enum class TripMode {
    GPS,
    REPLAY
}
